// Copyright 2014 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

library media_mojom;

import 'dart:async';

import 'package:mojo/bindings.dart' as bindings;
import 'package:mojo/core.dart' as core;



class _MediaPlayerPrepareParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  core.MojoDataPipeConsumer dataSource = null;

  _MediaPlayerPrepareParams() : super(kVersions.last.size);

  static _MediaPlayerPrepareParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static _MediaPlayerPrepareParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    _MediaPlayerPrepareParams result = new _MediaPlayerPrepareParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.dataSource = decoder0.decodeConsumerHandle(8, false);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeConsumerHandle(dataSource, 8, false);
  }

  String toString() {
    return "_MediaPlayerPrepareParams("
           "dataSource: $dataSource" ")";
  }

  Map toJson() {
    throw new bindings.MojoCodecError(
        'Object containing handles cannot be encoded to JSON.');
  }
}


class MediaPlayerPrepareResponseParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  bool ignored = false;

  MediaPlayerPrepareResponseParams() : super(kVersions.last.size);

  static MediaPlayerPrepareResponseParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static MediaPlayerPrepareResponseParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    MediaPlayerPrepareResponseParams result = new MediaPlayerPrepareResponseParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.ignored = decoder0.decodeBool(8, 0);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeBool(ignored, 8, 0);
  }

  String toString() {
    return "MediaPlayerPrepareResponseParams("
           "ignored: $ignored" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["ignored"] = ignored;
    return map;
  }
}


class _MediaPlayerStartParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(8, 0)
  ];

  _MediaPlayerStartParams() : super(kVersions.last.size);

  static _MediaPlayerStartParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static _MediaPlayerStartParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    _MediaPlayerStartParams result = new _MediaPlayerStartParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    encoder.getStructEncoderAtOffset(kVersions.last);
  }

  String toString() {
    return "_MediaPlayerStartParams("")";
  }

  Map toJson() {
    Map map = new Map();
    return map;
  }
}


class _MediaPlayerPauseParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(8, 0)
  ];

  _MediaPlayerPauseParams() : super(kVersions.last.size);

  static _MediaPlayerPauseParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static _MediaPlayerPauseParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    _MediaPlayerPauseParams result = new _MediaPlayerPauseParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    encoder.getStructEncoderAtOffset(kVersions.last);
  }

  String toString() {
    return "_MediaPlayerPauseParams("")";
  }

  Map toJson() {
    Map map = new Map();
    return map;
  }
}


class _MediaPlayerSeekToParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  int msec = 0;

  _MediaPlayerSeekToParams() : super(kVersions.last.size);

  static _MediaPlayerSeekToParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static _MediaPlayerSeekToParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    _MediaPlayerSeekToParams result = new _MediaPlayerSeekToParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.msec = decoder0.decodeUint32(8);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeUint32(msec, 8);
  }

  String toString() {
    return "_MediaPlayerSeekToParams("
           "msec: $msec" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["msec"] = msec;
    return map;
  }
}


class _MediaPlayerSetVolumeParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  double volume = 0.0;

  _MediaPlayerSetVolumeParams() : super(kVersions.last.size);

  static _MediaPlayerSetVolumeParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static _MediaPlayerSetVolumeParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    _MediaPlayerSetVolumeParams result = new _MediaPlayerSetVolumeParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.volume = decoder0.decodeFloat(8);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeFloat(volume, 8);
  }

  String toString() {
    return "_MediaPlayerSetVolumeParams("
           "volume: $volume" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["volume"] = volume;
    return map;
  }
}


class _MediaPlayerSetLoopingParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  bool looping = false;

  _MediaPlayerSetLoopingParams() : super(kVersions.last.size);

  static _MediaPlayerSetLoopingParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static _MediaPlayerSetLoopingParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    _MediaPlayerSetLoopingParams result = new _MediaPlayerSetLoopingParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.looping = decoder0.decodeBool(8, 0);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeBool(looping, 8, 0);
  }

  String toString() {
    return "_MediaPlayerSetLoopingParams("
           "looping: $looping" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["looping"] = looping;
    return map;
  }
}


class _SoundPoolLoadParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  core.MojoDataPipeConsumer dataSource = null;

  _SoundPoolLoadParams() : super(kVersions.last.size);

  static _SoundPoolLoadParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static _SoundPoolLoadParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    _SoundPoolLoadParams result = new _SoundPoolLoadParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.dataSource = decoder0.decodeConsumerHandle(8, false);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeConsumerHandle(dataSource, 8, false);
  }

  String toString() {
    return "_SoundPoolLoadParams("
           "dataSource: $dataSource" ")";
  }

  Map toJson() {
    throw new bindings.MojoCodecError(
        'Object containing handles cannot be encoded to JSON.');
  }
}


class SoundPoolLoadResponseParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  bool success = false;
  int soundId = 0;

  SoundPoolLoadResponseParams() : super(kVersions.last.size);

  static SoundPoolLoadResponseParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static SoundPoolLoadResponseParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    SoundPoolLoadResponseParams result = new SoundPoolLoadResponseParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.success = decoder0.decodeBool(8, 0);
    }
    if (mainDataHeader.version >= 0) {
      
      result.soundId = decoder0.decodeInt32(12);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeBool(success, 8, 0);
    
    encoder0.encodeInt32(soundId, 12);
  }

  String toString() {
    return "SoundPoolLoadResponseParams("
           "success: $success" ", "
           "soundId: $soundId" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["success"] = success;
    map["soundId"] = soundId;
    return map;
  }
}


class _SoundPoolPlayParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(32, 0)
  ];
  int soundId = 0;
  int streamId = 0;
  List<double> channelVolumes = null;
  bool loop = false;
  double rate = 0.0;

  _SoundPoolPlayParams() : super(kVersions.last.size);

  static _SoundPoolPlayParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static _SoundPoolPlayParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    _SoundPoolPlayParams result = new _SoundPoolPlayParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.soundId = decoder0.decodeInt32(8);
    }
    if (mainDataHeader.version >= 0) {
      
      result.streamId = decoder0.decodeInt32(12);
    }
    if (mainDataHeader.version >= 0) {
      
      result.channelVolumes = decoder0.decodeFloatArray(16, bindings.kNothingNullable, bindings.kUnspecifiedArrayLength);
    }
    if (mainDataHeader.version >= 0) {
      
      result.loop = decoder0.decodeBool(24, 0);
    }
    if (mainDataHeader.version >= 0) {
      
      result.rate = decoder0.decodeFloat(28);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeInt32(soundId, 8);
    
    encoder0.encodeInt32(streamId, 12);
    
    encoder0.encodeFloatArray(channelVolumes, 16, bindings.kNothingNullable, bindings.kUnspecifiedArrayLength);
    
    encoder0.encodeBool(loop, 24, 0);
    
    encoder0.encodeFloat(rate, 28);
  }

  String toString() {
    return "_SoundPoolPlayParams("
           "soundId: $soundId" ", "
           "streamId: $streamId" ", "
           "channelVolumes: $channelVolumes" ", "
           "loop: $loop" ", "
           "rate: $rate" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["soundId"] = soundId;
    map["streamId"] = streamId;
    map["channelVolumes"] = channelVolumes;
    map["loop"] = loop;
    map["rate"] = rate;
    return map;
  }
}


class SoundPoolPlayResponseParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  bool success = false;

  SoundPoolPlayResponseParams() : super(kVersions.last.size);

  static SoundPoolPlayResponseParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static SoundPoolPlayResponseParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    SoundPoolPlayResponseParams result = new SoundPoolPlayResponseParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.success = decoder0.decodeBool(8, 0);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeBool(success, 8, 0);
  }

  String toString() {
    return "SoundPoolPlayResponseParams("
           "success: $success" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["success"] = success;
    return map;
  }
}


class _SoundPoolStopParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  int streamId = 0;

  _SoundPoolStopParams() : super(kVersions.last.size);

  static _SoundPoolStopParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static _SoundPoolStopParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    _SoundPoolStopParams result = new _SoundPoolStopParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.streamId = decoder0.decodeInt32(8);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeInt32(streamId, 8);
  }

  String toString() {
    return "_SoundPoolStopParams("
           "streamId: $streamId" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["streamId"] = streamId;
    return map;
  }
}


class _SoundPoolPauseParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  int streamId = 0;

  _SoundPoolPauseParams() : super(kVersions.last.size);

  static _SoundPoolPauseParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static _SoundPoolPauseParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    _SoundPoolPauseParams result = new _SoundPoolPauseParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.streamId = decoder0.decodeInt32(8);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeInt32(streamId, 8);
  }

  String toString() {
    return "_SoundPoolPauseParams("
           "streamId: $streamId" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["streamId"] = streamId;
    return map;
  }
}


class _SoundPoolResumeParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  int streamId = 0;

  _SoundPoolResumeParams() : super(kVersions.last.size);

  static _SoundPoolResumeParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static _SoundPoolResumeParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    _SoundPoolResumeParams result = new _SoundPoolResumeParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.streamId = decoder0.decodeInt32(8);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeInt32(streamId, 8);
  }

  String toString() {
    return "_SoundPoolResumeParams("
           "streamId: $streamId" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["streamId"] = streamId;
    return map;
  }
}


class _SoundPoolSetRateParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  int streamId = 0;
  double rate = 0.0;

  _SoundPoolSetRateParams() : super(kVersions.last.size);

  static _SoundPoolSetRateParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static _SoundPoolSetRateParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    _SoundPoolSetRateParams result = new _SoundPoolSetRateParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.streamId = decoder0.decodeInt32(8);
    }
    if (mainDataHeader.version >= 0) {
      
      result.rate = decoder0.decodeFloat(12);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeInt32(streamId, 8);
    
    encoder0.encodeFloat(rate, 12);
  }

  String toString() {
    return "_SoundPoolSetRateParams("
           "streamId: $streamId" ", "
           "rate: $rate" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["streamId"] = streamId;
    map["rate"] = rate;
    return map;
  }
}


class _SoundPoolSetVolumeParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(24, 0)
  ];
  int streamId = 0;
  List<double> channelVolumes = null;

  _SoundPoolSetVolumeParams() : super(kVersions.last.size);

  static _SoundPoolSetVolumeParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static _SoundPoolSetVolumeParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    _SoundPoolSetVolumeParams result = new _SoundPoolSetVolumeParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.streamId = decoder0.decodeInt32(8);
    }
    if (mainDataHeader.version >= 0) {
      
      result.channelVolumes = decoder0.decodeFloatArray(16, bindings.kNothingNullable, bindings.kUnspecifiedArrayLength);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeInt32(streamId, 8);
    
    encoder0.encodeFloatArray(channelVolumes, 16, bindings.kNothingNullable, bindings.kUnspecifiedArrayLength);
  }

  String toString() {
    return "_SoundPoolSetVolumeParams("
           "streamId: $streamId" ", "
           "channelVolumes: $channelVolumes" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["streamId"] = streamId;
    map["channelVolumes"] = channelVolumes;
    return map;
  }
}


class _SoundPoolPauseAllParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(8, 0)
  ];

  _SoundPoolPauseAllParams() : super(kVersions.last.size);

  static _SoundPoolPauseAllParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static _SoundPoolPauseAllParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    _SoundPoolPauseAllParams result = new _SoundPoolPauseAllParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    encoder.getStructEncoderAtOffset(kVersions.last);
  }

  String toString() {
    return "_SoundPoolPauseAllParams("")";
  }

  Map toJson() {
    Map map = new Map();
    return map;
  }
}


class _SoundPoolResumeAllParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(8, 0)
  ];

  _SoundPoolResumeAllParams() : super(kVersions.last.size);

  static _SoundPoolResumeAllParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static _SoundPoolResumeAllParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    _SoundPoolResumeAllParams result = new _SoundPoolResumeAllParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    encoder.getStructEncoderAtOffset(kVersions.last);
  }

  String toString() {
    return "_SoundPoolResumeAllParams("")";
  }

  Map toJson() {
    Map map = new Map();
    return map;
  }
}


class _MediaServiceCreatePlayerParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  Object player = null;

  _MediaServiceCreatePlayerParams() : super(kVersions.last.size);

  static _MediaServiceCreatePlayerParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static _MediaServiceCreatePlayerParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    _MediaServiceCreatePlayerParams result = new _MediaServiceCreatePlayerParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.player = decoder0.decodeInterfaceRequest(8, false, MediaPlayerStub.newFromEndpoint);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeInterfaceRequest(player, 8, false);
  }

  String toString() {
    return "_MediaServiceCreatePlayerParams("
           "player: $player" ")";
  }

  Map toJson() {
    throw new bindings.MojoCodecError(
        'Object containing handles cannot be encoded to JSON.');
  }
}


class _MediaServiceCreateSoundPoolParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  Object pool = null;
  int maxStreams = 0;

  _MediaServiceCreateSoundPoolParams() : super(kVersions.last.size);

  static _MediaServiceCreateSoundPoolParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static _MediaServiceCreateSoundPoolParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    _MediaServiceCreateSoundPoolParams result = new _MediaServiceCreateSoundPoolParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.pool = decoder0.decodeInterfaceRequest(8, false, SoundPoolStub.newFromEndpoint);
    }
    if (mainDataHeader.version >= 0) {
      
      result.maxStreams = decoder0.decodeInt32(12);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeInterfaceRequest(pool, 8, false);
    
    encoder0.encodeInt32(maxStreams, 12);
  }

  String toString() {
    return "_MediaServiceCreateSoundPoolParams("
           "pool: $pool" ", "
           "maxStreams: $maxStreams" ")";
  }

  Map toJson() {
    throw new bindings.MojoCodecError(
        'Object containing handles cannot be encoded to JSON.');
  }
}

const int _MediaPlayer_prepareName = 0;
const int _MediaPlayer_startName = 1;
const int _MediaPlayer_pauseName = 2;
const int _MediaPlayer_seekToName = 3;
const int _MediaPlayer_setVolumeName = 4;
const int _MediaPlayer_setLoopingName = 5;

abstract class MediaPlayer {
  static const String serviceName = "media::MediaPlayer";
  dynamic prepare(core.MojoDataPipeConsumer dataSource,[Function responseFactory = null]);
  void start();
  void pause();
  void seekTo(int msec);
  void setVolume(double volume);
  void setLooping(bool looping);
}


class _MediaPlayerProxyImpl extends bindings.Proxy {
  _MediaPlayerProxyImpl.fromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) : super.fromEndpoint(endpoint);

  _MediaPlayerProxyImpl.fromHandle(core.MojoHandle handle) :
      super.fromHandle(handle);

  _MediaPlayerProxyImpl.unbound() : super.unbound();

  static _MediaPlayerProxyImpl newFromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) {
    assert(endpoint.setDescription("For _MediaPlayerProxyImpl"));
    return new _MediaPlayerProxyImpl.fromEndpoint(endpoint);
  }

  void handleResponse(bindings.ServiceMessage message) {
    switch (message.header.type) {
      case _MediaPlayer_prepareName:
        var r = MediaPlayerPrepareResponseParams.deserialize(
            message.payload);
        if (!message.header.hasRequestId) {
          proxyError("Expected a message with a valid request Id.");
          return;
        }
        Completer c = completerMap[message.header.requestId];
        if (c == null) {
          proxyError(
              "Message had unknown request Id: ${message.header.requestId}");
          return;
        }
        completerMap.remove(message.header.requestId);
        if (c.isCompleted) {
          proxyError("Response completer already completed");
          return;
        }
        c.complete(r);
        break;
      default:
        proxyError("Unexpected message type: ${message.header.type}");
        close(immediate: true);
        break;
    }
  }

  String toString() {
    var superString = super.toString();
    return "_MediaPlayerProxyImpl($superString)";
  }
}


class _MediaPlayerProxyCalls implements MediaPlayer {
  _MediaPlayerProxyImpl _proxyImpl;

  _MediaPlayerProxyCalls(this._proxyImpl);
    dynamic prepare(core.MojoDataPipeConsumer dataSource,[Function responseFactory = null]) {
      var params = new _MediaPlayerPrepareParams();
      params.dataSource = dataSource;
      return _proxyImpl.sendMessageWithRequestId(
          params,
          _MediaPlayer_prepareName,
          -1,
          bindings.MessageHeader.kMessageExpectsResponse);
    }
    void start() {
      if (!_proxyImpl.isBound) {
        _proxyImpl.proxyError("The Proxy is closed.");
        return;
      }
      var params = new _MediaPlayerStartParams();
      _proxyImpl.sendMessage(params, _MediaPlayer_startName);
    }
    void pause() {
      if (!_proxyImpl.isBound) {
        _proxyImpl.proxyError("The Proxy is closed.");
        return;
      }
      var params = new _MediaPlayerPauseParams();
      _proxyImpl.sendMessage(params, _MediaPlayer_pauseName);
    }
    void seekTo(int msec) {
      if (!_proxyImpl.isBound) {
        _proxyImpl.proxyError("The Proxy is closed.");
        return;
      }
      var params = new _MediaPlayerSeekToParams();
      params.msec = msec;
      _proxyImpl.sendMessage(params, _MediaPlayer_seekToName);
    }
    void setVolume(double volume) {
      if (!_proxyImpl.isBound) {
        _proxyImpl.proxyError("The Proxy is closed.");
        return;
      }
      var params = new _MediaPlayerSetVolumeParams();
      params.volume = volume;
      _proxyImpl.sendMessage(params, _MediaPlayer_setVolumeName);
    }
    void setLooping(bool looping) {
      if (!_proxyImpl.isBound) {
        _proxyImpl.proxyError("The Proxy is closed.");
        return;
      }
      var params = new _MediaPlayerSetLoopingParams();
      params.looping = looping;
      _proxyImpl.sendMessage(params, _MediaPlayer_setLoopingName);
    }
}


class MediaPlayerProxy implements bindings.ProxyBase {
  final bindings.Proxy impl;
  MediaPlayer ptr;

  MediaPlayerProxy(_MediaPlayerProxyImpl proxyImpl) :
      impl = proxyImpl,
      ptr = new _MediaPlayerProxyCalls(proxyImpl);

  MediaPlayerProxy.fromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) :
      impl = new _MediaPlayerProxyImpl.fromEndpoint(endpoint) {
    ptr = new _MediaPlayerProxyCalls(impl);
  }

  MediaPlayerProxy.fromHandle(core.MojoHandle handle) :
      impl = new _MediaPlayerProxyImpl.fromHandle(handle) {
    ptr = new _MediaPlayerProxyCalls(impl);
  }

  MediaPlayerProxy.unbound() :
      impl = new _MediaPlayerProxyImpl.unbound() {
    ptr = new _MediaPlayerProxyCalls(impl);
  }

  factory MediaPlayerProxy.connectToService(
      bindings.ServiceConnector s, String url, [String serviceName]) {
    MediaPlayerProxy p = new MediaPlayerProxy.unbound();
    s.connectToService(url, p, serviceName);
    return p;
  }

  static MediaPlayerProxy newFromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) {
    assert(endpoint.setDescription("For MediaPlayerProxy"));
    return new MediaPlayerProxy.fromEndpoint(endpoint);
  }

  String get serviceName => MediaPlayer.serviceName;

  Future close({bool immediate: false}) => impl.close(immediate: immediate);

  Future responseOrError(Future f) => impl.responseOrError(f);

  Future get errorFuture => impl.errorFuture;

  int get version => impl.version;

  Future<int> queryVersion() => impl.queryVersion();

  void requireVersion(int requiredVersion) {
    impl.requireVersion(requiredVersion);
  }

  String toString() {
    return "MediaPlayerProxy($impl)";
  }
}


class MediaPlayerStub extends bindings.Stub {
  MediaPlayer _impl = null;

  MediaPlayerStub.fromEndpoint(
      core.MojoMessagePipeEndpoint endpoint, [this._impl])
      : super.fromEndpoint(endpoint);

  MediaPlayerStub.fromHandle(core.MojoHandle handle, [this._impl])
      : super.fromHandle(handle);

  MediaPlayerStub.unbound() : super.unbound();

  static MediaPlayerStub newFromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) {
    assert(endpoint.setDescription("For MediaPlayerStub"));
    return new MediaPlayerStub.fromEndpoint(endpoint);
  }


  MediaPlayerPrepareResponseParams _MediaPlayerPrepareResponseParamsFactory(bool ignored) {
    var mojo_factory_result = new MediaPlayerPrepareResponseParams();
    mojo_factory_result.ignored = ignored;
    return mojo_factory_result;
  }

  dynamic handleMessage(bindings.ServiceMessage message) {
    if (bindings.ControlMessageHandler.isControlMessage(message)) {
      return bindings.ControlMessageHandler.handleMessage(this,
                                                          0,
                                                          message);
    }
    assert(_impl != null);
    switch (message.header.type) {
      case _MediaPlayer_prepareName:
        var params = _MediaPlayerPrepareParams.deserialize(
            message.payload);
        var response = _impl.prepare(params.dataSource,_MediaPlayerPrepareResponseParamsFactory);
        if (response is Future) {
          return response.then((response) {
            if (response != null) {
              return buildResponseWithId(
                  response,
                  _MediaPlayer_prepareName,
                  message.header.requestId,
                  bindings.MessageHeader.kMessageIsResponse);
            }
          });
        } else if (response != null) {
          return buildResponseWithId(
              response,
              _MediaPlayer_prepareName,
              message.header.requestId,
              bindings.MessageHeader.kMessageIsResponse);
        }
        break;
      case _MediaPlayer_startName:
        var params = _MediaPlayerStartParams.deserialize(
            message.payload);
        _impl.start();
        break;
      case _MediaPlayer_pauseName:
        var params = _MediaPlayerPauseParams.deserialize(
            message.payload);
        _impl.pause();
        break;
      case _MediaPlayer_seekToName:
        var params = _MediaPlayerSeekToParams.deserialize(
            message.payload);
        _impl.seekTo(params.msec);
        break;
      case _MediaPlayer_setVolumeName:
        var params = _MediaPlayerSetVolumeParams.deserialize(
            message.payload);
        _impl.setVolume(params.volume);
        break;
      case _MediaPlayer_setLoopingName:
        var params = _MediaPlayerSetLoopingParams.deserialize(
            message.payload);
        _impl.setLooping(params.looping);
        break;
      default:
        throw new bindings.MojoCodecError("Unexpected message name");
        break;
    }
    return null;
  }

  MediaPlayer get impl => _impl;
  set impl(MediaPlayer d) {
    assert(_impl == null);
    _impl = d;
  }

  String toString() {
    var superString = super.toString();
    return "MediaPlayerStub($superString)";
  }

  int get version => 0;
}

const int _SoundPool_loadName = 0;
const int _SoundPool_playName = 1;
const int _SoundPool_stopName = 2;
const int _SoundPool_pauseName = 3;
const int _SoundPool_resumeName = 4;
const int _SoundPool_setRateName = 5;
const int _SoundPool_setVolumeName = 6;
const int _SoundPool_pauseAllName = 7;
const int _SoundPool_resumeAllName = 8;

abstract class SoundPool {
  static const String serviceName = "media::SoundPool";
  dynamic load(core.MojoDataPipeConsumer dataSource,[Function responseFactory = null]);
  dynamic play(int soundId,int streamId,List<double> channelVolumes,bool loop,double rate,[Function responseFactory = null]);
  void stop(int streamId);
  void pause(int streamId);
  void resume(int streamId);
  void setRate(int streamId, double rate);
  void setVolume(int streamId, List<double> channelVolumes);
  void pauseAll();
  void resumeAll();
}


class _SoundPoolProxyImpl extends bindings.Proxy {
  _SoundPoolProxyImpl.fromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) : super.fromEndpoint(endpoint);

  _SoundPoolProxyImpl.fromHandle(core.MojoHandle handle) :
      super.fromHandle(handle);

  _SoundPoolProxyImpl.unbound() : super.unbound();

  static _SoundPoolProxyImpl newFromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) {
    assert(endpoint.setDescription("For _SoundPoolProxyImpl"));
    return new _SoundPoolProxyImpl.fromEndpoint(endpoint);
  }

  void handleResponse(bindings.ServiceMessage message) {
    switch (message.header.type) {
      case _SoundPool_loadName:
        var r = SoundPoolLoadResponseParams.deserialize(
            message.payload);
        if (!message.header.hasRequestId) {
          proxyError("Expected a message with a valid request Id.");
          return;
        }
        Completer c = completerMap[message.header.requestId];
        if (c == null) {
          proxyError(
              "Message had unknown request Id: ${message.header.requestId}");
          return;
        }
        completerMap.remove(message.header.requestId);
        if (c.isCompleted) {
          proxyError("Response completer already completed");
          return;
        }
        c.complete(r);
        break;
      case _SoundPool_playName:
        var r = SoundPoolPlayResponseParams.deserialize(
            message.payload);
        if (!message.header.hasRequestId) {
          proxyError("Expected a message with a valid request Id.");
          return;
        }
        Completer c = completerMap[message.header.requestId];
        if (c == null) {
          proxyError(
              "Message had unknown request Id: ${message.header.requestId}");
          return;
        }
        completerMap.remove(message.header.requestId);
        if (c.isCompleted) {
          proxyError("Response completer already completed");
          return;
        }
        c.complete(r);
        break;
      default:
        proxyError("Unexpected message type: ${message.header.type}");
        close(immediate: true);
        break;
    }
  }

  String toString() {
    var superString = super.toString();
    return "_SoundPoolProxyImpl($superString)";
  }
}


class _SoundPoolProxyCalls implements SoundPool {
  _SoundPoolProxyImpl _proxyImpl;

  _SoundPoolProxyCalls(this._proxyImpl);
    dynamic load(core.MojoDataPipeConsumer dataSource,[Function responseFactory = null]) {
      var params = new _SoundPoolLoadParams();
      params.dataSource = dataSource;
      return _proxyImpl.sendMessageWithRequestId(
          params,
          _SoundPool_loadName,
          -1,
          bindings.MessageHeader.kMessageExpectsResponse);
    }
    dynamic play(int soundId,int streamId,List<double> channelVolumes,bool loop,double rate,[Function responseFactory = null]) {
      var params = new _SoundPoolPlayParams();
      params.soundId = soundId;
      params.streamId = streamId;
      params.channelVolumes = channelVolumes;
      params.loop = loop;
      params.rate = rate;
      return _proxyImpl.sendMessageWithRequestId(
          params,
          _SoundPool_playName,
          -1,
          bindings.MessageHeader.kMessageExpectsResponse);
    }
    void stop(int streamId) {
      if (!_proxyImpl.isBound) {
        _proxyImpl.proxyError("The Proxy is closed.");
        return;
      }
      var params = new _SoundPoolStopParams();
      params.streamId = streamId;
      _proxyImpl.sendMessage(params, _SoundPool_stopName);
    }
    void pause(int streamId) {
      if (!_proxyImpl.isBound) {
        _proxyImpl.proxyError("The Proxy is closed.");
        return;
      }
      var params = new _SoundPoolPauseParams();
      params.streamId = streamId;
      _proxyImpl.sendMessage(params, _SoundPool_pauseName);
    }
    void resume(int streamId) {
      if (!_proxyImpl.isBound) {
        _proxyImpl.proxyError("The Proxy is closed.");
        return;
      }
      var params = new _SoundPoolResumeParams();
      params.streamId = streamId;
      _proxyImpl.sendMessage(params, _SoundPool_resumeName);
    }
    void setRate(int streamId, double rate) {
      if (!_proxyImpl.isBound) {
        _proxyImpl.proxyError("The Proxy is closed.");
        return;
      }
      var params = new _SoundPoolSetRateParams();
      params.streamId = streamId;
      params.rate = rate;
      _proxyImpl.sendMessage(params, _SoundPool_setRateName);
    }
    void setVolume(int streamId, List<double> channelVolumes) {
      if (!_proxyImpl.isBound) {
        _proxyImpl.proxyError("The Proxy is closed.");
        return;
      }
      var params = new _SoundPoolSetVolumeParams();
      params.streamId = streamId;
      params.channelVolumes = channelVolumes;
      _proxyImpl.sendMessage(params, _SoundPool_setVolumeName);
    }
    void pauseAll() {
      if (!_proxyImpl.isBound) {
        _proxyImpl.proxyError("The Proxy is closed.");
        return;
      }
      var params = new _SoundPoolPauseAllParams();
      _proxyImpl.sendMessage(params, _SoundPool_pauseAllName);
    }
    void resumeAll() {
      if (!_proxyImpl.isBound) {
        _proxyImpl.proxyError("The Proxy is closed.");
        return;
      }
      var params = new _SoundPoolResumeAllParams();
      _proxyImpl.sendMessage(params, _SoundPool_resumeAllName);
    }
}


class SoundPoolProxy implements bindings.ProxyBase {
  final bindings.Proxy impl;
  SoundPool ptr;

  SoundPoolProxy(_SoundPoolProxyImpl proxyImpl) :
      impl = proxyImpl,
      ptr = new _SoundPoolProxyCalls(proxyImpl);

  SoundPoolProxy.fromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) :
      impl = new _SoundPoolProxyImpl.fromEndpoint(endpoint) {
    ptr = new _SoundPoolProxyCalls(impl);
  }

  SoundPoolProxy.fromHandle(core.MojoHandle handle) :
      impl = new _SoundPoolProxyImpl.fromHandle(handle) {
    ptr = new _SoundPoolProxyCalls(impl);
  }

  SoundPoolProxy.unbound() :
      impl = new _SoundPoolProxyImpl.unbound() {
    ptr = new _SoundPoolProxyCalls(impl);
  }

  factory SoundPoolProxy.connectToService(
      bindings.ServiceConnector s, String url, [String serviceName]) {
    SoundPoolProxy p = new SoundPoolProxy.unbound();
    s.connectToService(url, p, serviceName);
    return p;
  }

  static SoundPoolProxy newFromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) {
    assert(endpoint.setDescription("For SoundPoolProxy"));
    return new SoundPoolProxy.fromEndpoint(endpoint);
  }

  String get serviceName => SoundPool.serviceName;

  Future close({bool immediate: false}) => impl.close(immediate: immediate);

  Future responseOrError(Future f) => impl.responseOrError(f);

  Future get errorFuture => impl.errorFuture;

  int get version => impl.version;

  Future<int> queryVersion() => impl.queryVersion();

  void requireVersion(int requiredVersion) {
    impl.requireVersion(requiredVersion);
  }

  String toString() {
    return "SoundPoolProxy($impl)";
  }
}


class SoundPoolStub extends bindings.Stub {
  SoundPool _impl = null;

  SoundPoolStub.fromEndpoint(
      core.MojoMessagePipeEndpoint endpoint, [this._impl])
      : super.fromEndpoint(endpoint);

  SoundPoolStub.fromHandle(core.MojoHandle handle, [this._impl])
      : super.fromHandle(handle);

  SoundPoolStub.unbound() : super.unbound();

  static SoundPoolStub newFromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) {
    assert(endpoint.setDescription("For SoundPoolStub"));
    return new SoundPoolStub.fromEndpoint(endpoint);
  }


  SoundPoolLoadResponseParams _SoundPoolLoadResponseParamsFactory(bool success, int soundId) {
    var mojo_factory_result = new SoundPoolLoadResponseParams();
    mojo_factory_result.success = success;
    mojo_factory_result.soundId = soundId;
    return mojo_factory_result;
  }
  SoundPoolPlayResponseParams _SoundPoolPlayResponseParamsFactory(bool success) {
    var mojo_factory_result = new SoundPoolPlayResponseParams();
    mojo_factory_result.success = success;
    return mojo_factory_result;
  }

  dynamic handleMessage(bindings.ServiceMessage message) {
    if (bindings.ControlMessageHandler.isControlMessage(message)) {
      return bindings.ControlMessageHandler.handleMessage(this,
                                                          0,
                                                          message);
    }
    assert(_impl != null);
    switch (message.header.type) {
      case _SoundPool_loadName:
        var params = _SoundPoolLoadParams.deserialize(
            message.payload);
        var response = _impl.load(params.dataSource,_SoundPoolLoadResponseParamsFactory);
        if (response is Future) {
          return response.then((response) {
            if (response != null) {
              return buildResponseWithId(
                  response,
                  _SoundPool_loadName,
                  message.header.requestId,
                  bindings.MessageHeader.kMessageIsResponse);
            }
          });
        } else if (response != null) {
          return buildResponseWithId(
              response,
              _SoundPool_loadName,
              message.header.requestId,
              bindings.MessageHeader.kMessageIsResponse);
        }
        break;
      case _SoundPool_playName:
        var params = _SoundPoolPlayParams.deserialize(
            message.payload);
        var response = _impl.play(params.soundId,params.streamId,params.channelVolumes,params.loop,params.rate,_SoundPoolPlayResponseParamsFactory);
        if (response is Future) {
          return response.then((response) {
            if (response != null) {
              return buildResponseWithId(
                  response,
                  _SoundPool_playName,
                  message.header.requestId,
                  bindings.MessageHeader.kMessageIsResponse);
            }
          });
        } else if (response != null) {
          return buildResponseWithId(
              response,
              _SoundPool_playName,
              message.header.requestId,
              bindings.MessageHeader.kMessageIsResponse);
        }
        break;
      case _SoundPool_stopName:
        var params = _SoundPoolStopParams.deserialize(
            message.payload);
        _impl.stop(params.streamId);
        break;
      case _SoundPool_pauseName:
        var params = _SoundPoolPauseParams.deserialize(
            message.payload);
        _impl.pause(params.streamId);
        break;
      case _SoundPool_resumeName:
        var params = _SoundPoolResumeParams.deserialize(
            message.payload);
        _impl.resume(params.streamId);
        break;
      case _SoundPool_setRateName:
        var params = _SoundPoolSetRateParams.deserialize(
            message.payload);
        _impl.setRate(params.streamId, params.rate);
        break;
      case _SoundPool_setVolumeName:
        var params = _SoundPoolSetVolumeParams.deserialize(
            message.payload);
        _impl.setVolume(params.streamId, params.channelVolumes);
        break;
      case _SoundPool_pauseAllName:
        var params = _SoundPoolPauseAllParams.deserialize(
            message.payload);
        _impl.pauseAll();
        break;
      case _SoundPool_resumeAllName:
        var params = _SoundPoolResumeAllParams.deserialize(
            message.payload);
        _impl.resumeAll();
        break;
      default:
        throw new bindings.MojoCodecError("Unexpected message name");
        break;
    }
    return null;
  }

  SoundPool get impl => _impl;
  set impl(SoundPool d) {
    assert(_impl == null);
    _impl = d;
  }

  String toString() {
    var superString = super.toString();
    return "SoundPoolStub($superString)";
  }

  int get version => 0;
}

const int _MediaService_createPlayerName = 0;
const int _MediaService_createSoundPoolName = 1;

abstract class MediaService {
  static const String serviceName = "media::MediaService";
  void createPlayer(Object player);
  void createSoundPool(Object pool, int maxStreams);
}


class _MediaServiceProxyImpl extends bindings.Proxy {
  _MediaServiceProxyImpl.fromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) : super.fromEndpoint(endpoint);

  _MediaServiceProxyImpl.fromHandle(core.MojoHandle handle) :
      super.fromHandle(handle);

  _MediaServiceProxyImpl.unbound() : super.unbound();

  static _MediaServiceProxyImpl newFromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) {
    assert(endpoint.setDescription("For _MediaServiceProxyImpl"));
    return new _MediaServiceProxyImpl.fromEndpoint(endpoint);
  }

  void handleResponse(bindings.ServiceMessage message) {
    switch (message.header.type) {
      default:
        proxyError("Unexpected message type: ${message.header.type}");
        close(immediate: true);
        break;
    }
  }

  String toString() {
    var superString = super.toString();
    return "_MediaServiceProxyImpl($superString)";
  }
}


class _MediaServiceProxyCalls implements MediaService {
  _MediaServiceProxyImpl _proxyImpl;

  _MediaServiceProxyCalls(this._proxyImpl);
    void createPlayer(Object player) {
      if (!_proxyImpl.isBound) {
        _proxyImpl.proxyError("The Proxy is closed.");
        return;
      }
      var params = new _MediaServiceCreatePlayerParams();
      params.player = player;
      _proxyImpl.sendMessage(params, _MediaService_createPlayerName);
    }
    void createSoundPool(Object pool, int maxStreams) {
      if (!_proxyImpl.isBound) {
        _proxyImpl.proxyError("The Proxy is closed.");
        return;
      }
      var params = new _MediaServiceCreateSoundPoolParams();
      params.pool = pool;
      params.maxStreams = maxStreams;
      _proxyImpl.sendMessage(params, _MediaService_createSoundPoolName);
    }
}


class MediaServiceProxy implements bindings.ProxyBase {
  final bindings.Proxy impl;
  MediaService ptr;

  MediaServiceProxy(_MediaServiceProxyImpl proxyImpl) :
      impl = proxyImpl,
      ptr = new _MediaServiceProxyCalls(proxyImpl);

  MediaServiceProxy.fromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) :
      impl = new _MediaServiceProxyImpl.fromEndpoint(endpoint) {
    ptr = new _MediaServiceProxyCalls(impl);
  }

  MediaServiceProxy.fromHandle(core.MojoHandle handle) :
      impl = new _MediaServiceProxyImpl.fromHandle(handle) {
    ptr = new _MediaServiceProxyCalls(impl);
  }

  MediaServiceProxy.unbound() :
      impl = new _MediaServiceProxyImpl.unbound() {
    ptr = new _MediaServiceProxyCalls(impl);
  }

  factory MediaServiceProxy.connectToService(
      bindings.ServiceConnector s, String url, [String serviceName]) {
    MediaServiceProxy p = new MediaServiceProxy.unbound();
    s.connectToService(url, p, serviceName);
    return p;
  }

  static MediaServiceProxy newFromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) {
    assert(endpoint.setDescription("For MediaServiceProxy"));
    return new MediaServiceProxy.fromEndpoint(endpoint);
  }

  String get serviceName => MediaService.serviceName;

  Future close({bool immediate: false}) => impl.close(immediate: immediate);

  Future responseOrError(Future f) => impl.responseOrError(f);

  Future get errorFuture => impl.errorFuture;

  int get version => impl.version;

  Future<int> queryVersion() => impl.queryVersion();

  void requireVersion(int requiredVersion) {
    impl.requireVersion(requiredVersion);
  }

  String toString() {
    return "MediaServiceProxy($impl)";
  }
}


class MediaServiceStub extends bindings.Stub {
  MediaService _impl = null;

  MediaServiceStub.fromEndpoint(
      core.MojoMessagePipeEndpoint endpoint, [this._impl])
      : super.fromEndpoint(endpoint);

  MediaServiceStub.fromHandle(core.MojoHandle handle, [this._impl])
      : super.fromHandle(handle);

  MediaServiceStub.unbound() : super.unbound();

  static MediaServiceStub newFromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) {
    assert(endpoint.setDescription("For MediaServiceStub"));
    return new MediaServiceStub.fromEndpoint(endpoint);
  }



  dynamic handleMessage(bindings.ServiceMessage message) {
    if (bindings.ControlMessageHandler.isControlMessage(message)) {
      return bindings.ControlMessageHandler.handleMessage(this,
                                                          0,
                                                          message);
    }
    assert(_impl != null);
    switch (message.header.type) {
      case _MediaService_createPlayerName:
        var params = _MediaServiceCreatePlayerParams.deserialize(
            message.payload);
        _impl.createPlayer(params.player);
        break;
      case _MediaService_createSoundPoolName:
        var params = _MediaServiceCreateSoundPoolParams.deserialize(
            message.payload);
        _impl.createSoundPool(params.pool, params.maxStreams);
        break;
      default:
        throw new bindings.MojoCodecError("Unexpected message name");
        break;
    }
    return null;
  }

  MediaService get impl => _impl;
  set impl(MediaService d) {
    assert(_impl == null);
    _impl = d;
  }

  String toString() {
    var superString = super.toString();
    return "MediaServiceStub($superString)";
  }

  int get version => 0;
}


