// Copyright 2014 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

library pointer_mojom;

import 'dart:async';

import 'package:mojo/bindings.dart' as bindings;
import 'package:mojo/core.dart' as core;

class PointerType extends bindings.MojoEnum {
  static const PointerType down = const PointerType._(0);
  static const PointerType up = const PointerType._(1);
  static const PointerType move = const PointerType._(2);
  static const PointerType cancel = const PointerType._(3);

  const PointerType._(int v) : super(v);

  static const Map<String, PointerType> valuesMap = const {
    "down": down,
    "up": up,
    "move": move,
    "cancel": cancel,
  };
  static const List<PointerType> values = const [
    down,
    up,
    move,
    cancel,
  ];

  static PointerType valueOf(String name) => valuesMap[name];

  factory PointerType(int v) {
    switch (v) {
      case 0:
        return down;
      case 1:
        return up;
      case 2:
        return move;
      case 3:
        return cancel;
      default:
        return null;
    }
  }

  static PointerType decode(bindings.Decoder decoder0, int offset) {
    int v = decoder0.decodeUint32(offset);
    PointerType result = new PointerType(v);
    if (result == null) {
      throw new bindings.MojoCodecError(
          'Bad value $v for enum PointerType.');
    }
    return result;
  }

  String toString() {
    switch(this) {
      case down:
        return 'PointerType.down';
      case up:
        return 'PointerType.up';
      case move:
        return 'PointerType.move';
      case cancel:
        return 'PointerType.cancel';
    }
  }

  int toJson() => mojoEnumValue;
}

class PointerKind extends bindings.MojoEnum {
  static const PointerKind touch = const PointerKind._(0);
  static const PointerKind mouse = const PointerKind._(1);
  static const PointerKind stylus = const PointerKind._(2);
  static const PointerKind invertedStylus = const PointerKind._(3);

  const PointerKind._(int v) : super(v);

  static const Map<String, PointerKind> valuesMap = const {
    "touch": touch,
    "mouse": mouse,
    "stylus": stylus,
    "invertedStylus": invertedStylus,
  };
  static const List<PointerKind> values = const [
    touch,
    mouse,
    stylus,
    invertedStylus,
  ];

  static PointerKind valueOf(String name) => valuesMap[name];

  factory PointerKind(int v) {
    switch (v) {
      case 0:
        return touch;
      case 1:
        return mouse;
      case 2:
        return stylus;
      case 3:
        return invertedStylus;
      default:
        return null;
    }
  }

  static PointerKind decode(bindings.Decoder decoder0, int offset) {
    int v = decoder0.decodeUint32(offset);
    PointerKind result = new PointerKind(v);
    if (result == null) {
      throw new bindings.MojoCodecError(
          'Bad value $v for enum PointerKind.');
    }
    return result;
  }

  String toString() {
    switch(this) {
      case touch:
        return 'PointerKind.touch';
      case mouse:
        return 'PointerKind.mouse';
      case stylus:
        return 'PointerKind.stylus';
      case invertedStylus:
        return 'PointerKind.invertedStylus';
    }
  }

  int toJson() => mojoEnumValue;
}



class Pointer extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(96, 0)
  ];
  int timeStamp = 0;
  int pointer = 0;
  PointerType type = null;
  PointerKind kind = null;
  double x = 0.0;
  double y = 0.0;
  int buttons = 0;
  bool down = false;
  bool primary = false;
  bool obscured = false;
  double pressure = 0.0;
  double pressureMin = 0.0;
  double pressureMax = 0.0;
  double distance = 0.0;
  double distanceMin = 0.0;
  double distanceMax = 0.0;
  double radiusMajor = 0.0;
  double radiusMinor = 0.0;
  double radiusMin = 0.0;
  double radiusMax = 0.0;
  double orientation = 0.0;
  double tilt = 0.0;

  Pointer() : super(kVersions.last.size);

  static Pointer deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static Pointer decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    Pointer result = new Pointer();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.timeStamp = decoder0.decodeInt64(8);
    }
    if (mainDataHeader.version >= 0) {
      
      result.pointer = decoder0.decodeInt32(16);
    }
    if (mainDataHeader.version >= 0) {
      
        result.type = PointerType.decode(decoder0, 20);
        if (result.type == null) {
          throw new bindings.MojoCodecError(
            'Trying to decode null union for non-nullable PointerType.');
        }
    }
    if (mainDataHeader.version >= 0) {
      
        result.kind = PointerKind.decode(decoder0, 24);
        if (result.kind == null) {
          throw new bindings.MojoCodecError(
            'Trying to decode null union for non-nullable PointerKind.');
        }
    }
    if (mainDataHeader.version >= 0) {
      
      result.x = decoder0.decodeFloat(28);
    }
    if (mainDataHeader.version >= 0) {
      
      result.y = decoder0.decodeFloat(32);
    }
    if (mainDataHeader.version >= 0) {
      
      result.buttons = decoder0.decodeInt32(36);
    }
    if (mainDataHeader.version >= 0) {
      
      result.down = decoder0.decodeBool(40, 0);
    }
    if (mainDataHeader.version >= 0) {
      
      result.primary = decoder0.decodeBool(40, 1);
    }
    if (mainDataHeader.version >= 0) {
      
      result.obscured = decoder0.decodeBool(40, 2);
    }
    if (mainDataHeader.version >= 0) {
      
      result.pressure = decoder0.decodeFloat(44);
    }
    if (mainDataHeader.version >= 0) {
      
      result.pressureMin = decoder0.decodeFloat(48);
    }
    if (mainDataHeader.version >= 0) {
      
      result.pressureMax = decoder0.decodeFloat(52);
    }
    if (mainDataHeader.version >= 0) {
      
      result.distance = decoder0.decodeFloat(56);
    }
    if (mainDataHeader.version >= 0) {
      
      result.distanceMin = decoder0.decodeFloat(60);
    }
    if (mainDataHeader.version >= 0) {
      
      result.distanceMax = decoder0.decodeFloat(64);
    }
    if (mainDataHeader.version >= 0) {
      
      result.radiusMajor = decoder0.decodeFloat(68);
    }
    if (mainDataHeader.version >= 0) {
      
      result.radiusMinor = decoder0.decodeFloat(72);
    }
    if (mainDataHeader.version >= 0) {
      
      result.radiusMin = decoder0.decodeFloat(76);
    }
    if (mainDataHeader.version >= 0) {
      
      result.radiusMax = decoder0.decodeFloat(80);
    }
    if (mainDataHeader.version >= 0) {
      
      result.orientation = decoder0.decodeFloat(84);
    }
    if (mainDataHeader.version >= 0) {
      
      result.tilt = decoder0.decodeFloat(88);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeInt64(timeStamp, 8);
    
    encoder0.encodeInt32(pointer, 16);
    
    encoder0.encodeEnum(type, 20);
    
    encoder0.encodeEnum(kind, 24);
    
    encoder0.encodeFloat(x, 28);
    
    encoder0.encodeFloat(y, 32);
    
    encoder0.encodeInt32(buttons, 36);
    
    encoder0.encodeBool(down, 40, 0);
    
    encoder0.encodeBool(primary, 40, 1);
    
    encoder0.encodeBool(obscured, 40, 2);
    
    encoder0.encodeFloat(pressure, 44);
    
    encoder0.encodeFloat(pressureMin, 48);
    
    encoder0.encodeFloat(pressureMax, 52);
    
    encoder0.encodeFloat(distance, 56);
    
    encoder0.encodeFloat(distanceMin, 60);
    
    encoder0.encodeFloat(distanceMax, 64);
    
    encoder0.encodeFloat(radiusMajor, 68);
    
    encoder0.encodeFloat(radiusMinor, 72);
    
    encoder0.encodeFloat(radiusMin, 76);
    
    encoder0.encodeFloat(radiusMax, 80);
    
    encoder0.encodeFloat(orientation, 84);
    
    encoder0.encodeFloat(tilt, 88);
  }

  String toString() {
    return "Pointer("
           "timeStamp: $timeStamp" ", "
           "pointer: $pointer" ", "
           "type: $type" ", "
           "kind: $kind" ", "
           "x: $x" ", "
           "y: $y" ", "
           "buttons: $buttons" ", "
           "down: $down" ", "
           "primary: $primary" ", "
           "obscured: $obscured" ", "
           "pressure: $pressure" ", "
           "pressureMin: $pressureMin" ", "
           "pressureMax: $pressureMax" ", "
           "distance: $distance" ", "
           "distanceMin: $distanceMin" ", "
           "distanceMax: $distanceMax" ", "
           "radiusMajor: $radiusMajor" ", "
           "radiusMinor: $radiusMinor" ", "
           "radiusMin: $radiusMin" ", "
           "radiusMax: $radiusMax" ", "
           "orientation: $orientation" ", "
           "tilt: $tilt" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["timeStamp"] = timeStamp;
    map["pointer"] = pointer;
    map["type"] = type;
    map["kind"] = kind;
    map["x"] = x;
    map["y"] = y;
    map["buttons"] = buttons;
    map["down"] = down;
    map["primary"] = primary;
    map["obscured"] = obscured;
    map["pressure"] = pressure;
    map["pressureMin"] = pressureMin;
    map["pressureMax"] = pressureMax;
    map["distance"] = distance;
    map["distanceMin"] = distanceMin;
    map["distanceMax"] = distanceMax;
    map["radiusMajor"] = radiusMajor;
    map["radiusMinor"] = radiusMinor;
    map["radiusMin"] = radiusMin;
    map["radiusMax"] = radiusMax;
    map["orientation"] = orientation;
    map["tilt"] = tilt;
    return map;
  }
}


class PointerPacket extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  List<Pointer> pointers = null;

  PointerPacket() : super(kVersions.last.size);

  static PointerPacket deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static PointerPacket decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    PointerPacket result = new PointerPacket();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      var decoder1 = decoder0.decodePointer(8, false);
      {
        var si1 = decoder1.decodeDataHeaderForPointerArray(bindings.kUnspecifiedArrayLength);
        result.pointers = new List<Pointer>(si1.numElements);
        for (int i1 = 0; i1 < si1.numElements; ++i1) {
          
          var decoder2 = decoder1.decodePointer(bindings.ArrayDataHeader.kHeaderSize + bindings.kPointerSize * i1, false);
          result.pointers[i1] = Pointer.decode(decoder2);
        }
      }
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    if (pointers == null) {
      encoder0.encodeNullPointer(8, false);
    } else {
      var encoder1 = encoder0.encodePointerArray(pointers.length, 8, bindings.kUnspecifiedArrayLength);
      for (int i0 = 0; i0 < pointers.length; ++i0) {
        
        encoder1.encodeStruct(pointers[i0], bindings.ArrayDataHeader.kHeaderSize + bindings.kPointerSize * i0, false);
      }
    }
  }

  String toString() {
    return "PointerPacket("
           "pointers: $pointers" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["pointers"] = pointers;
    return map;
  }
}


class _PointerListenerOnPointerPacketParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  PointerPacket packet = null;

  _PointerListenerOnPointerPacketParams() : super(kVersions.last.size);

  static _PointerListenerOnPointerPacketParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static _PointerListenerOnPointerPacketParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    _PointerListenerOnPointerPacketParams result = new _PointerListenerOnPointerPacketParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      var decoder1 = decoder0.decodePointer(8, false);
      result.packet = PointerPacket.decode(decoder1);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeStruct(packet, 8, false);
  }

  String toString() {
    return "_PointerListenerOnPointerPacketParams("
           "packet: $packet" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["packet"] = packet;
    return map;
  }
}

const int _PointerListener_onPointerPacketName = 0;

abstract class PointerListener {
  static const String serviceName = "pointer::PointerListener";
  void onPointerPacket(PointerPacket packet);
}


class _PointerListenerProxyImpl extends bindings.Proxy {
  _PointerListenerProxyImpl.fromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) : super.fromEndpoint(endpoint);

  _PointerListenerProxyImpl.fromHandle(core.MojoHandle handle) :
      super.fromHandle(handle);

  _PointerListenerProxyImpl.unbound() : super.unbound();

  static _PointerListenerProxyImpl newFromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) {
    assert(endpoint.setDescription("For _PointerListenerProxyImpl"));
    return new _PointerListenerProxyImpl.fromEndpoint(endpoint);
  }

  void handleResponse(bindings.ServiceMessage message) {
    switch (message.header.type) {
      default:
        proxyError("Unexpected message type: ${message.header.type}");
        close(immediate: true);
        break;
    }
  }

  String toString() {
    var superString = super.toString();
    return "_PointerListenerProxyImpl($superString)";
  }
}


class _PointerListenerProxyCalls implements PointerListener {
  _PointerListenerProxyImpl _proxyImpl;

  _PointerListenerProxyCalls(this._proxyImpl);
    void onPointerPacket(PointerPacket packet) {
      if (!_proxyImpl.isBound) {
        _proxyImpl.proxyError("The Proxy is closed.");
        return;
      }
      var params = new _PointerListenerOnPointerPacketParams();
      params.packet = packet;
      _proxyImpl.sendMessage(params, _PointerListener_onPointerPacketName);
    }
}


class PointerListenerProxy implements bindings.ProxyBase {
  final bindings.Proxy impl;
  PointerListener ptr;

  PointerListenerProxy(_PointerListenerProxyImpl proxyImpl) :
      impl = proxyImpl,
      ptr = new _PointerListenerProxyCalls(proxyImpl);

  PointerListenerProxy.fromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) :
      impl = new _PointerListenerProxyImpl.fromEndpoint(endpoint) {
    ptr = new _PointerListenerProxyCalls(impl);
  }

  PointerListenerProxy.fromHandle(core.MojoHandle handle) :
      impl = new _PointerListenerProxyImpl.fromHandle(handle) {
    ptr = new _PointerListenerProxyCalls(impl);
  }

  PointerListenerProxy.unbound() :
      impl = new _PointerListenerProxyImpl.unbound() {
    ptr = new _PointerListenerProxyCalls(impl);
  }

  factory PointerListenerProxy.connectToService(
      bindings.ServiceConnector s, String url, [String serviceName]) {
    PointerListenerProxy p = new PointerListenerProxy.unbound();
    s.connectToService(url, p, serviceName);
    return p;
  }

  static PointerListenerProxy newFromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) {
    assert(endpoint.setDescription("For PointerListenerProxy"));
    return new PointerListenerProxy.fromEndpoint(endpoint);
  }

  String get serviceName => PointerListener.serviceName;

  Future close({bool immediate: false}) => impl.close(immediate: immediate);

  Future responseOrError(Future f) => impl.responseOrError(f);

  Future get errorFuture => impl.errorFuture;

  int get version => impl.version;

  Future<int> queryVersion() => impl.queryVersion();

  void requireVersion(int requiredVersion) {
    impl.requireVersion(requiredVersion);
  }

  String toString() {
    return "PointerListenerProxy($impl)";
  }
}


class PointerListenerStub extends bindings.Stub {
  PointerListener _impl = null;

  PointerListenerStub.fromEndpoint(
      core.MojoMessagePipeEndpoint endpoint, [this._impl])
      : super.fromEndpoint(endpoint);

  PointerListenerStub.fromHandle(core.MojoHandle handle, [this._impl])
      : super.fromHandle(handle);

  PointerListenerStub.unbound() : super.unbound();

  static PointerListenerStub newFromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) {
    assert(endpoint.setDescription("For PointerListenerStub"));
    return new PointerListenerStub.fromEndpoint(endpoint);
  }



  dynamic handleMessage(bindings.ServiceMessage message) {
    if (bindings.ControlMessageHandler.isControlMessage(message)) {
      return bindings.ControlMessageHandler.handleMessage(this,
                                                          0,
                                                          message);
    }
    assert(_impl != null);
    switch (message.header.type) {
      case _PointerListener_onPointerPacketName:
        var params = _PointerListenerOnPointerPacketParams.deserialize(
            message.payload);
        _impl.onPointerPacket(params.packet);
        break;
      default:
        throw new bindings.MojoCodecError("Unexpected message name");
        break;
    }
    return null;
  }

  PointerListener get impl => _impl;
  set impl(PointerListener d) {
    assert(_impl == null);
    _impl = d;
  }

  String toString() {
    var superString = super.toString();
    return "PointerListenerStub($superString)";
  }

  int get version => 0;
}


