// Copyright 2014 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

library semantics_mojom;

import 'dart:async';

import 'package:mojo/bindings.dart' as bindings;
import 'package:mojo/core.dart' as core;



class SemanticsNode extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(48, 0)
  ];
  int id = 0;
  SemanticFlags flags = null;
  SemanticStrings strings = null;
  SemanticGeometry geometry = null;
  List<SemanticsNode> children = null;

  SemanticsNode() : super(kVersions.last.size);

  static SemanticsNode deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static SemanticsNode decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    SemanticsNode result = new SemanticsNode();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.id = decoder0.decodeUint32(8);
    }
    if (mainDataHeader.version >= 0) {
      
      var decoder1 = decoder0.decodePointer(16, true);
      result.flags = SemanticFlags.decode(decoder1);
    }
    if (mainDataHeader.version >= 0) {
      
      var decoder1 = decoder0.decodePointer(24, true);
      result.strings = SemanticStrings.decode(decoder1);
    }
    if (mainDataHeader.version >= 0) {
      
      var decoder1 = decoder0.decodePointer(32, true);
      result.geometry = SemanticGeometry.decode(decoder1);
    }
    if (mainDataHeader.version >= 0) {
      
      var decoder1 = decoder0.decodePointer(40, false);
      {
        var si1 = decoder1.decodeDataHeaderForPointerArray(bindings.kUnspecifiedArrayLength);
        result.children = new List<SemanticsNode>(si1.numElements);
        for (int i1 = 0; i1 < si1.numElements; ++i1) {
          
          var decoder2 = decoder1.decodePointer(bindings.ArrayDataHeader.kHeaderSize + bindings.kPointerSize * i1, false);
          result.children[i1] = SemanticsNode.decode(decoder2);
        }
      }
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeUint32(id, 8);
    
    encoder0.encodeStruct(flags, 16, true);
    
    encoder0.encodeStruct(strings, 24, true);
    
    encoder0.encodeStruct(geometry, 32, true);
    
    if (children == null) {
      encoder0.encodeNullPointer(40, false);
    } else {
      var encoder1 = encoder0.encodePointerArray(children.length, 40, bindings.kUnspecifiedArrayLength);
      for (int i0 = 0; i0 < children.length; ++i0) {
        
        encoder1.encodeStruct(children[i0], bindings.ArrayDataHeader.kHeaderSize + bindings.kPointerSize * i0, false);
      }
    }
  }

  String toString() {
    return "SemanticsNode("
           "id: $id" ", "
           "flags: $flags" ", "
           "strings: $strings" ", "
           "geometry: $geometry" ", "
           "children: $children" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["id"] = id;
    map["flags"] = flags;
    map["strings"] = strings;
    map["geometry"] = geometry;
    map["children"] = children;
    return map;
  }
}


class SemanticFlags extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  bool canBeTapped = false;
  bool canBeLongPressed = false;
  bool canBeScrolledHorizontally = false;
  bool canBeScrolledVertically = false;
  bool hasCheckedState = false;
  bool isChecked = false;

  SemanticFlags() : super(kVersions.last.size);

  static SemanticFlags deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static SemanticFlags decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    SemanticFlags result = new SemanticFlags();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.canBeTapped = decoder0.decodeBool(8, 0);
    }
    if (mainDataHeader.version >= 0) {
      
      result.canBeLongPressed = decoder0.decodeBool(8, 1);
    }
    if (mainDataHeader.version >= 0) {
      
      result.canBeScrolledHorizontally = decoder0.decodeBool(8, 2);
    }
    if (mainDataHeader.version >= 0) {
      
      result.canBeScrolledVertically = decoder0.decodeBool(8, 3);
    }
    if (mainDataHeader.version >= 0) {
      
      result.hasCheckedState = decoder0.decodeBool(8, 4);
    }
    if (mainDataHeader.version >= 0) {
      
      result.isChecked = decoder0.decodeBool(8, 5);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeBool(canBeTapped, 8, 0);
    
    encoder0.encodeBool(canBeLongPressed, 8, 1);
    
    encoder0.encodeBool(canBeScrolledHorizontally, 8, 2);
    
    encoder0.encodeBool(canBeScrolledVertically, 8, 3);
    
    encoder0.encodeBool(hasCheckedState, 8, 4);
    
    encoder0.encodeBool(isChecked, 8, 5);
  }

  String toString() {
    return "SemanticFlags("
           "canBeTapped: $canBeTapped" ", "
           "canBeLongPressed: $canBeLongPressed" ", "
           "canBeScrolledHorizontally: $canBeScrolledHorizontally" ", "
           "canBeScrolledVertically: $canBeScrolledVertically" ", "
           "hasCheckedState: $hasCheckedState" ", "
           "isChecked: $isChecked" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["canBeTapped"] = canBeTapped;
    map["canBeLongPressed"] = canBeLongPressed;
    map["canBeScrolledHorizontally"] = canBeScrolledHorizontally;
    map["canBeScrolledVertically"] = canBeScrolledVertically;
    map["hasCheckedState"] = hasCheckedState;
    map["isChecked"] = isChecked;
    return map;
  }
}


class SemanticStrings extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  String label = null;

  SemanticStrings() : super(kVersions.last.size);

  static SemanticStrings deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static SemanticStrings decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    SemanticStrings result = new SemanticStrings();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.label = decoder0.decodeString(8, false);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeString(label, 8, false);
  }

  String toString() {
    return "SemanticStrings("
           "label: $label" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["label"] = label;
    return map;
  }
}


class SemanticGeometry extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(32, 0)
  ];
  List<double> transform = null;
  double left = 0.0;
  double top = 0.0;
  double width = 0.0;
  double height = 0.0;

  SemanticGeometry() : super(kVersions.last.size);

  static SemanticGeometry deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static SemanticGeometry decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    SemanticGeometry result = new SemanticGeometry();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.transform = decoder0.decodeFloatArray(8, bindings.kNothingNullable, bindings.kUnspecifiedArrayLength);
    }
    if (mainDataHeader.version >= 0) {
      
      result.left = decoder0.decodeFloat(16);
    }
    if (mainDataHeader.version >= 0) {
      
      result.top = decoder0.decodeFloat(20);
    }
    if (mainDataHeader.version >= 0) {
      
      result.width = decoder0.decodeFloat(24);
    }
    if (mainDataHeader.version >= 0) {
      
      result.height = decoder0.decodeFloat(28);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeFloatArray(transform, 8, bindings.kNothingNullable, bindings.kUnspecifiedArrayLength);
    
    encoder0.encodeFloat(left, 16);
    
    encoder0.encodeFloat(top, 20);
    
    encoder0.encodeFloat(width, 24);
    
    encoder0.encodeFloat(height, 28);
  }

  String toString() {
    return "SemanticGeometry("
           "transform: $transform" ", "
           "left: $left" ", "
           "top: $top" ", "
           "width: $width" ", "
           "height: $height" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["transform"] = transform;
    map["left"] = left;
    map["top"] = top;
    map["width"] = width;
    map["height"] = height;
    return map;
  }
}


class _SemanticsListenerUpdateSemanticsTreeParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  List<SemanticsNode> nodes = null;

  _SemanticsListenerUpdateSemanticsTreeParams() : super(kVersions.last.size);

  static _SemanticsListenerUpdateSemanticsTreeParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static _SemanticsListenerUpdateSemanticsTreeParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    _SemanticsListenerUpdateSemanticsTreeParams result = new _SemanticsListenerUpdateSemanticsTreeParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      var decoder1 = decoder0.decodePointer(8, false);
      {
        var si1 = decoder1.decodeDataHeaderForPointerArray(bindings.kUnspecifiedArrayLength);
        result.nodes = new List<SemanticsNode>(si1.numElements);
        for (int i1 = 0; i1 < si1.numElements; ++i1) {
          
          var decoder2 = decoder1.decodePointer(bindings.ArrayDataHeader.kHeaderSize + bindings.kPointerSize * i1, false);
          result.nodes[i1] = SemanticsNode.decode(decoder2);
        }
      }
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    if (nodes == null) {
      encoder0.encodeNullPointer(8, false);
    } else {
      var encoder1 = encoder0.encodePointerArray(nodes.length, 8, bindings.kUnspecifiedArrayLength);
      for (int i0 = 0; i0 < nodes.length; ++i0) {
        
        encoder1.encodeStruct(nodes[i0], bindings.ArrayDataHeader.kHeaderSize + bindings.kPointerSize * i0, false);
      }
    }
  }

  String toString() {
    return "_SemanticsListenerUpdateSemanticsTreeParams("
           "nodes: $nodes" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["nodes"] = nodes;
    return map;
  }
}


class _SemanticsServerAddSemanticsListenerParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  Object listener = null;

  _SemanticsServerAddSemanticsListenerParams() : super(kVersions.last.size);

  static _SemanticsServerAddSemanticsListenerParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static _SemanticsServerAddSemanticsListenerParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    _SemanticsServerAddSemanticsListenerParams result = new _SemanticsServerAddSemanticsListenerParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.listener = decoder0.decodeServiceInterface(8, false, SemanticsListenerProxy.newFromEndpoint);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeInterface(listener, 8, false);
  }

  String toString() {
    return "_SemanticsServerAddSemanticsListenerParams("
           "listener: $listener" ")";
  }

  Map toJson() {
    throw new bindings.MojoCodecError(
        'Object containing handles cannot be encoded to JSON.');
  }
}


class _SemanticsServerTapParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  int nodeId = 0;

  _SemanticsServerTapParams() : super(kVersions.last.size);

  static _SemanticsServerTapParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static _SemanticsServerTapParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    _SemanticsServerTapParams result = new _SemanticsServerTapParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.nodeId = decoder0.decodeUint32(8);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeUint32(nodeId, 8);
  }

  String toString() {
    return "_SemanticsServerTapParams("
           "nodeId: $nodeId" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["nodeId"] = nodeId;
    return map;
  }
}


class _SemanticsServerLongPressParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  int nodeId = 0;

  _SemanticsServerLongPressParams() : super(kVersions.last.size);

  static _SemanticsServerLongPressParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static _SemanticsServerLongPressParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    _SemanticsServerLongPressParams result = new _SemanticsServerLongPressParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.nodeId = decoder0.decodeUint32(8);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeUint32(nodeId, 8);
  }

  String toString() {
    return "_SemanticsServerLongPressParams("
           "nodeId: $nodeId" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["nodeId"] = nodeId;
    return map;
  }
}


class _SemanticsServerScrollLeftParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  int nodeId = 0;

  _SemanticsServerScrollLeftParams() : super(kVersions.last.size);

  static _SemanticsServerScrollLeftParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static _SemanticsServerScrollLeftParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    _SemanticsServerScrollLeftParams result = new _SemanticsServerScrollLeftParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.nodeId = decoder0.decodeUint32(8);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeUint32(nodeId, 8);
  }

  String toString() {
    return "_SemanticsServerScrollLeftParams("
           "nodeId: $nodeId" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["nodeId"] = nodeId;
    return map;
  }
}


class _SemanticsServerScrollRightParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  int nodeId = 0;

  _SemanticsServerScrollRightParams() : super(kVersions.last.size);

  static _SemanticsServerScrollRightParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static _SemanticsServerScrollRightParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    _SemanticsServerScrollRightParams result = new _SemanticsServerScrollRightParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.nodeId = decoder0.decodeUint32(8);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeUint32(nodeId, 8);
  }

  String toString() {
    return "_SemanticsServerScrollRightParams("
           "nodeId: $nodeId" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["nodeId"] = nodeId;
    return map;
  }
}


class _SemanticsServerScrollUpParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  int nodeId = 0;

  _SemanticsServerScrollUpParams() : super(kVersions.last.size);

  static _SemanticsServerScrollUpParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static _SemanticsServerScrollUpParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    _SemanticsServerScrollUpParams result = new _SemanticsServerScrollUpParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.nodeId = decoder0.decodeUint32(8);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeUint32(nodeId, 8);
  }

  String toString() {
    return "_SemanticsServerScrollUpParams("
           "nodeId: $nodeId" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["nodeId"] = nodeId;
    return map;
  }
}


class _SemanticsServerScrollDownParams extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(16, 0)
  ];
  int nodeId = 0;

  _SemanticsServerScrollDownParams() : super(kVersions.last.size);

  static _SemanticsServerScrollDownParams deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static _SemanticsServerScrollDownParams decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    _SemanticsServerScrollDownParams result = new _SemanticsServerScrollDownParams();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.nodeId = decoder0.decodeUint32(8);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeUint32(nodeId, 8);
  }

  String toString() {
    return "_SemanticsServerScrollDownParams("
           "nodeId: $nodeId" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["nodeId"] = nodeId;
    return map;
  }
}

const int _SemanticsListener_updateSemanticsTreeName = 0;

abstract class SemanticsListener {
  static const String serviceName = "semantics::SemanticsClient";
  void updateSemanticsTree(List<SemanticsNode> nodes);
}


class _SemanticsListenerProxyImpl extends bindings.Proxy {
  _SemanticsListenerProxyImpl.fromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) : super.fromEndpoint(endpoint);

  _SemanticsListenerProxyImpl.fromHandle(core.MojoHandle handle) :
      super.fromHandle(handle);

  _SemanticsListenerProxyImpl.unbound() : super.unbound();

  static _SemanticsListenerProxyImpl newFromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) {
    assert(endpoint.setDescription("For _SemanticsListenerProxyImpl"));
    return new _SemanticsListenerProxyImpl.fromEndpoint(endpoint);
  }

  void handleResponse(bindings.ServiceMessage message) {
    switch (message.header.type) {
      default:
        proxyError("Unexpected message type: ${message.header.type}");
        close(immediate: true);
        break;
    }
  }

  String toString() {
    var superString = super.toString();
    return "_SemanticsListenerProxyImpl($superString)";
  }
}


class _SemanticsListenerProxyCalls implements SemanticsListener {
  _SemanticsListenerProxyImpl _proxyImpl;

  _SemanticsListenerProxyCalls(this._proxyImpl);
    void updateSemanticsTree(List<SemanticsNode> nodes) {
      if (!_proxyImpl.isBound) {
        _proxyImpl.proxyError("The Proxy is closed.");
        return;
      }
      var params = new _SemanticsListenerUpdateSemanticsTreeParams();
      params.nodes = nodes;
      _proxyImpl.sendMessage(params, _SemanticsListener_updateSemanticsTreeName);
    }
}


class SemanticsListenerProxy implements bindings.ProxyBase {
  final bindings.Proxy impl;
  SemanticsListener ptr;

  SemanticsListenerProxy(_SemanticsListenerProxyImpl proxyImpl) :
      impl = proxyImpl,
      ptr = new _SemanticsListenerProxyCalls(proxyImpl);

  SemanticsListenerProxy.fromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) :
      impl = new _SemanticsListenerProxyImpl.fromEndpoint(endpoint) {
    ptr = new _SemanticsListenerProxyCalls(impl);
  }

  SemanticsListenerProxy.fromHandle(core.MojoHandle handle) :
      impl = new _SemanticsListenerProxyImpl.fromHandle(handle) {
    ptr = new _SemanticsListenerProxyCalls(impl);
  }

  SemanticsListenerProxy.unbound() :
      impl = new _SemanticsListenerProxyImpl.unbound() {
    ptr = new _SemanticsListenerProxyCalls(impl);
  }

  factory SemanticsListenerProxy.connectToService(
      bindings.ServiceConnector s, String url, [String serviceName]) {
    SemanticsListenerProxy p = new SemanticsListenerProxy.unbound();
    s.connectToService(url, p, serviceName);
    return p;
  }

  static SemanticsListenerProxy newFromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) {
    assert(endpoint.setDescription("For SemanticsListenerProxy"));
    return new SemanticsListenerProxy.fromEndpoint(endpoint);
  }

  String get serviceName => SemanticsListener.serviceName;

  Future close({bool immediate: false}) => impl.close(immediate: immediate);

  Future responseOrError(Future f) => impl.responseOrError(f);

  Future get errorFuture => impl.errorFuture;

  int get version => impl.version;

  Future<int> queryVersion() => impl.queryVersion();

  void requireVersion(int requiredVersion) {
    impl.requireVersion(requiredVersion);
  }

  String toString() {
    return "SemanticsListenerProxy($impl)";
  }
}


class SemanticsListenerStub extends bindings.Stub {
  SemanticsListener _impl = null;

  SemanticsListenerStub.fromEndpoint(
      core.MojoMessagePipeEndpoint endpoint, [this._impl])
      : super.fromEndpoint(endpoint);

  SemanticsListenerStub.fromHandle(core.MojoHandle handle, [this._impl])
      : super.fromHandle(handle);

  SemanticsListenerStub.unbound() : super.unbound();

  static SemanticsListenerStub newFromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) {
    assert(endpoint.setDescription("For SemanticsListenerStub"));
    return new SemanticsListenerStub.fromEndpoint(endpoint);
  }



  dynamic handleMessage(bindings.ServiceMessage message) {
    if (bindings.ControlMessageHandler.isControlMessage(message)) {
      return bindings.ControlMessageHandler.handleMessage(this,
                                                          0,
                                                          message);
    }
    assert(_impl != null);
    switch (message.header.type) {
      case _SemanticsListener_updateSemanticsTreeName:
        var params = _SemanticsListenerUpdateSemanticsTreeParams.deserialize(
            message.payload);
        _impl.updateSemanticsTree(params.nodes);
        break;
      default:
        throw new bindings.MojoCodecError("Unexpected message name");
        break;
    }
    return null;
  }

  SemanticsListener get impl => _impl;
  set impl(SemanticsListener d) {
    assert(_impl == null);
    _impl = d;
  }

  String toString() {
    var superString = super.toString();
    return "SemanticsListenerStub($superString)";
  }

  int get version => 0;
}

const int _SemanticsServer_addSemanticsListenerName = 0;
const int _SemanticsServer_tapName = 1;
const int _SemanticsServer_longPressName = 2;
const int _SemanticsServer_scrollLeftName = 3;
const int _SemanticsServer_scrollRightName = 4;
const int _SemanticsServer_scrollUpName = 5;
const int _SemanticsServer_scrollDownName = 6;

abstract class SemanticsServer {
  static const String serviceName = "semantics::SemanticsServer";
  void addSemanticsListener(Object listener);
  void tap(int nodeId);
  void longPress(int nodeId);
  void scrollLeft(int nodeId);
  void scrollRight(int nodeId);
  void scrollUp(int nodeId);
  void scrollDown(int nodeId);
}


class _SemanticsServerProxyImpl extends bindings.Proxy {
  _SemanticsServerProxyImpl.fromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) : super.fromEndpoint(endpoint);

  _SemanticsServerProxyImpl.fromHandle(core.MojoHandle handle) :
      super.fromHandle(handle);

  _SemanticsServerProxyImpl.unbound() : super.unbound();

  static _SemanticsServerProxyImpl newFromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) {
    assert(endpoint.setDescription("For _SemanticsServerProxyImpl"));
    return new _SemanticsServerProxyImpl.fromEndpoint(endpoint);
  }

  void handleResponse(bindings.ServiceMessage message) {
    switch (message.header.type) {
      default:
        proxyError("Unexpected message type: ${message.header.type}");
        close(immediate: true);
        break;
    }
  }

  String toString() {
    var superString = super.toString();
    return "_SemanticsServerProxyImpl($superString)";
  }
}


class _SemanticsServerProxyCalls implements SemanticsServer {
  _SemanticsServerProxyImpl _proxyImpl;

  _SemanticsServerProxyCalls(this._proxyImpl);
    void addSemanticsListener(Object listener) {
      if (!_proxyImpl.isBound) {
        _proxyImpl.proxyError("The Proxy is closed.");
        return;
      }
      var params = new _SemanticsServerAddSemanticsListenerParams();
      params.listener = listener;
      _proxyImpl.sendMessage(params, _SemanticsServer_addSemanticsListenerName);
    }
    void tap(int nodeId) {
      if (!_proxyImpl.isBound) {
        _proxyImpl.proxyError("The Proxy is closed.");
        return;
      }
      var params = new _SemanticsServerTapParams();
      params.nodeId = nodeId;
      _proxyImpl.sendMessage(params, _SemanticsServer_tapName);
    }
    void longPress(int nodeId) {
      if (!_proxyImpl.isBound) {
        _proxyImpl.proxyError("The Proxy is closed.");
        return;
      }
      var params = new _SemanticsServerLongPressParams();
      params.nodeId = nodeId;
      _proxyImpl.sendMessage(params, _SemanticsServer_longPressName);
    }
    void scrollLeft(int nodeId) {
      if (!_proxyImpl.isBound) {
        _proxyImpl.proxyError("The Proxy is closed.");
        return;
      }
      var params = new _SemanticsServerScrollLeftParams();
      params.nodeId = nodeId;
      _proxyImpl.sendMessage(params, _SemanticsServer_scrollLeftName);
    }
    void scrollRight(int nodeId) {
      if (!_proxyImpl.isBound) {
        _proxyImpl.proxyError("The Proxy is closed.");
        return;
      }
      var params = new _SemanticsServerScrollRightParams();
      params.nodeId = nodeId;
      _proxyImpl.sendMessage(params, _SemanticsServer_scrollRightName);
    }
    void scrollUp(int nodeId) {
      if (!_proxyImpl.isBound) {
        _proxyImpl.proxyError("The Proxy is closed.");
        return;
      }
      var params = new _SemanticsServerScrollUpParams();
      params.nodeId = nodeId;
      _proxyImpl.sendMessage(params, _SemanticsServer_scrollUpName);
    }
    void scrollDown(int nodeId) {
      if (!_proxyImpl.isBound) {
        _proxyImpl.proxyError("The Proxy is closed.");
        return;
      }
      var params = new _SemanticsServerScrollDownParams();
      params.nodeId = nodeId;
      _proxyImpl.sendMessage(params, _SemanticsServer_scrollDownName);
    }
}


class SemanticsServerProxy implements bindings.ProxyBase {
  final bindings.Proxy impl;
  SemanticsServer ptr;

  SemanticsServerProxy(_SemanticsServerProxyImpl proxyImpl) :
      impl = proxyImpl,
      ptr = new _SemanticsServerProxyCalls(proxyImpl);

  SemanticsServerProxy.fromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) :
      impl = new _SemanticsServerProxyImpl.fromEndpoint(endpoint) {
    ptr = new _SemanticsServerProxyCalls(impl);
  }

  SemanticsServerProxy.fromHandle(core.MojoHandle handle) :
      impl = new _SemanticsServerProxyImpl.fromHandle(handle) {
    ptr = new _SemanticsServerProxyCalls(impl);
  }

  SemanticsServerProxy.unbound() :
      impl = new _SemanticsServerProxyImpl.unbound() {
    ptr = new _SemanticsServerProxyCalls(impl);
  }

  factory SemanticsServerProxy.connectToService(
      bindings.ServiceConnector s, String url, [String serviceName]) {
    SemanticsServerProxy p = new SemanticsServerProxy.unbound();
    s.connectToService(url, p, serviceName);
    return p;
  }

  static SemanticsServerProxy newFromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) {
    assert(endpoint.setDescription("For SemanticsServerProxy"));
    return new SemanticsServerProxy.fromEndpoint(endpoint);
  }

  String get serviceName => SemanticsServer.serviceName;

  Future close({bool immediate: false}) => impl.close(immediate: immediate);

  Future responseOrError(Future f) => impl.responseOrError(f);

  Future get errorFuture => impl.errorFuture;

  int get version => impl.version;

  Future<int> queryVersion() => impl.queryVersion();

  void requireVersion(int requiredVersion) {
    impl.requireVersion(requiredVersion);
  }

  String toString() {
    return "SemanticsServerProxy($impl)";
  }
}


class SemanticsServerStub extends bindings.Stub {
  SemanticsServer _impl = null;

  SemanticsServerStub.fromEndpoint(
      core.MojoMessagePipeEndpoint endpoint, [this._impl])
      : super.fromEndpoint(endpoint);

  SemanticsServerStub.fromHandle(core.MojoHandle handle, [this._impl])
      : super.fromHandle(handle);

  SemanticsServerStub.unbound() : super.unbound();

  static SemanticsServerStub newFromEndpoint(
      core.MojoMessagePipeEndpoint endpoint) {
    assert(endpoint.setDescription("For SemanticsServerStub"));
    return new SemanticsServerStub.fromEndpoint(endpoint);
  }



  dynamic handleMessage(bindings.ServiceMessage message) {
    if (bindings.ControlMessageHandler.isControlMessage(message)) {
      return bindings.ControlMessageHandler.handleMessage(this,
                                                          0,
                                                          message);
    }
    assert(_impl != null);
    switch (message.header.type) {
      case _SemanticsServer_addSemanticsListenerName:
        var params = _SemanticsServerAddSemanticsListenerParams.deserialize(
            message.payload);
        _impl.addSemanticsListener(params.listener);
        break;
      case _SemanticsServer_tapName:
        var params = _SemanticsServerTapParams.deserialize(
            message.payload);
        _impl.tap(params.nodeId);
        break;
      case _SemanticsServer_longPressName:
        var params = _SemanticsServerLongPressParams.deserialize(
            message.payload);
        _impl.longPress(params.nodeId);
        break;
      case _SemanticsServer_scrollLeftName:
        var params = _SemanticsServerScrollLeftParams.deserialize(
            message.payload);
        _impl.scrollLeft(params.nodeId);
        break;
      case _SemanticsServer_scrollRightName:
        var params = _SemanticsServerScrollRightParams.deserialize(
            message.payload);
        _impl.scrollRight(params.nodeId);
        break;
      case _SemanticsServer_scrollUpName:
        var params = _SemanticsServerScrollUpParams.deserialize(
            message.payload);
        _impl.scrollUp(params.nodeId);
        break;
      case _SemanticsServer_scrollDownName:
        var params = _SemanticsServerScrollDownParams.deserialize(
            message.payload);
        _impl.scrollDown(params.nodeId);
        break;
      default:
        throw new bindings.MojoCodecError("Unexpected message name");
        break;
    }
    return null;
  }

  SemanticsServer get impl => _impl;
  set impl(SemanticsServer d) {
    assert(_impl == null);
    _impl = d;
  }

  String toString() {
    var superString = super.toString();
    return "SemanticsServerStub($superString)";
  }

  int get version => 0;
}


