// Copyright 2014 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

library input_event_mojom;

import 'dart:async';

import 'package:mojo/bindings.dart' as bindings;
import 'package:mojo/core.dart' as core;

class EventType extends bindings.MojoEnum {
  static const EventType unknown = const EventType._(0);
  static const EventType back = const EventType._(1);
  static const EventType keyPressed = const EventType._(2);
  static const EventType keyReleased = const EventType._(3);

  const EventType._(int v) : super(v);

  static const Map<String, EventType> valuesMap = const {
    "unknown": unknown,
    "back": back,
    "keyPressed": keyPressed,
    "keyReleased": keyReleased,
  };
  static const List<EventType> values = const [
    unknown,
    back,
    keyPressed,
    keyReleased,
  ];

  static EventType valueOf(String name) => valuesMap[name];

  factory EventType(int v) {
    switch (v) {
      case 0:
        return unknown;
      case 1:
        return back;
      case 2:
        return keyPressed;
      case 3:
        return keyReleased;
      default:
        return null;
    }
  }

  static EventType decode(bindings.Decoder decoder0, int offset) {
    int v = decoder0.decodeUint32(offset);
    EventType result = new EventType(v);
    if (result == null) {
      throw new bindings.MojoCodecError(
          'Bad value $v for enum EventType.');
    }
    return result;
  }

  String toString() {
    switch(this) {
      case unknown:
        return 'EventType.unknown';
      case back:
        return 'EventType.back';
      case keyPressed:
        return 'EventType.keyPressed';
      case keyReleased:
        return 'EventType.keyReleased';
    }
  }

  int toJson() => mojoEnumValue;
}



class KeyData extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(24, 0)
  ];
  int keyCode = 0;
  int flags = 0;
  int scanCode = 0;
  int metaState = 0;

  KeyData() : super(kVersions.last.size);

  static KeyData deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static KeyData decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    KeyData result = new KeyData();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
      result.keyCode = decoder0.decodeInt32(8);
    }
    if (mainDataHeader.version >= 0) {
      
      result.flags = decoder0.decodeInt32(12);
    }
    if (mainDataHeader.version >= 0) {
      
      result.scanCode = decoder0.decodeInt32(16);
    }
    if (mainDataHeader.version >= 0) {
      
      result.metaState = decoder0.decodeInt32(20);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeInt32(keyCode, 8);
    
    encoder0.encodeInt32(flags, 12);
    
    encoder0.encodeInt32(scanCode, 16);
    
    encoder0.encodeInt32(metaState, 20);
  }

  String toString() {
    return "KeyData("
           "keyCode: $keyCode" ", "
           "flags: $flags" ", "
           "scanCode: $scanCode" ", "
           "metaState: $metaState" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["keyCode"] = keyCode;
    map["flags"] = flags;
    map["scanCode"] = scanCode;
    map["metaState"] = metaState;
    return map;
  }
}


class InputEvent extends bindings.Struct {
  static const List<bindings.StructDataHeader> kVersions = const [
    const bindings.StructDataHeader(32, 0)
  ];
  EventType type = null;
  int timeStamp = 0;
  KeyData keyData = null;

  InputEvent() : super(kVersions.last.size);

  static InputEvent deserialize(bindings.Message message) {
    var decoder = new bindings.Decoder(message);
    var result = decode(decoder);
    if (decoder.excessHandles != null) {
      decoder.excessHandles.forEach((h) => h.close());
    }
    return result;
  }

  static InputEvent decode(bindings.Decoder decoder0) {
    if (decoder0 == null) {
      return null;
    }
    InputEvent result = new InputEvent();

    var mainDataHeader = decoder0.decodeStructDataHeader();
    if (mainDataHeader.version <= kVersions.last.version) {
      // Scan in reverse order to optimize for more recent versions.
      for (int i = kVersions.length - 1; i >= 0; --i) {
        if (mainDataHeader.version >= kVersions[i].version) {
          if (mainDataHeader.size == kVersions[i].size) {
            // Found a match.
            break;
          }
          throw new bindings.MojoCodecError(
              'Header size doesn\'t correspond to known version size.');
        }
      }
    } else if (mainDataHeader.size < kVersions.last.size) {
      throw new bindings.MojoCodecError(
        'Message newer than the last known version cannot be shorter than '
        'required by the last known version.');
    }
    if (mainDataHeader.version >= 0) {
      
        result.type = EventType.decode(decoder0, 8);
        if (result.type == null) {
          throw new bindings.MojoCodecError(
            'Trying to decode null union for non-nullable EventType.');
        }
    }
    if (mainDataHeader.version >= 0) {
      
      result.timeStamp = decoder0.decodeInt64(16);
    }
    if (mainDataHeader.version >= 0) {
      
      var decoder1 = decoder0.decodePointer(24, true);
      result.keyData = KeyData.decode(decoder1);
    }
    return result;
  }

  void encode(bindings.Encoder encoder) {
    var encoder0 = encoder.getStructEncoderAtOffset(kVersions.last);
    
    encoder0.encodeEnum(type, 8);
    
    encoder0.encodeInt64(timeStamp, 16);
    
    encoder0.encodeStruct(keyData, 24, true);
  }

  String toString() {
    return "InputEvent("
           "type: $type" ", "
           "timeStamp: $timeStamp" ", "
           "keyData: $keyData" ")";
  }

  Map toJson() {
    Map map = new Map();
    map["type"] = type;
    map["timeStamp"] = timeStamp;
    map["keyData"] = keyData;
    return map;
  }
}


